/*  *************************************************************************

	Shadow Door - a Neverwinter Nights NPC control interface
	Copyright 2003 by Robert Zubek and Phillip Saltzman, Northwestern University
    email: rob@cs.northwestern.edu, p-saltzman@northwestern.edu

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

************************************************************************** */


//
//
// pointer table class

#include "stdafx.h"

// constructor
PointerTable::PointerTable () 
{
	root = new Node();
	hAccessLock = CreateMutex (NULL, FALSE, "PointerTableMutex");
}

// destructor 
PointerTable::~PointerTable ()
{
	// deallocate the entire dictionary
	delete root;
}

// adding an entry
void PointerTable::AddEntry (char *name, ShadowDoorFnPtr pointer, Node *node) 
{
	char letter = name[0];
	char *rest = &(name[1]);

	// if we reached end of string, add the pointer to the current node
	if (letter == '\0') 
	{
		node->AddPointer (pointer);
	}

	// otherwise we still have recursion to perform
	else 
	{
		// is there a next node set up under the current letter? 
		// if not, make one and link it.
		Node *next = node->GetNode (letter);
		if (next == NULL) {
			next = new Node();
			node->AddNode (letter, next);
			_ASSERTE (next == node->GetNode (letter));
		}

		// now recurse
		AddEntry (rest, pointer, next);
	}
}


void PointerTable::AddEntry (char *name, ShadowDoorFnPtr function) 
{
	DWORD dResult = WaitForSingleObject (hAccessLock, INFINITE);
	if (dResult == WAIT_OBJECT_0) 
	{
        AddEntry (name, function, root);
		ReleaseMutex (hAccessLock);
	}
}


// retrieving an entry
ShadowDoorFnPtr PointerTable::GetEntry (char *name, Node *node) 
{
	char letter = name[0];
	char *rest = &(name[1]);

	// if we reached the end of string, return what's there
	if (letter == NULL) return node->GetPointer();

	// see if we can go deeper in the dictionary
	Node *next = node->GetNode (letter);
	if (next == NULL) return NULL;

	// and recurse
	else return GetEntry (rest, next);
}

ShadowDoorFnPtr PointerTable::GetEntry (char *name) 
{
	ShadowDoorFnPtr p = NULL;
	DWORD dResult = WaitForSingleObject (hAccessLock, INFINITE);
	if (dResult == WAIT_OBJECT_0) 
	{
        p = GetEntry (name, root);
		ReleaseMutex (hAccessLock);
	}
	return p;
}

