/*  *************************************************************************

	Shadow Door - a Neverwinter Nights NPC control interface
	Copyright 2003 by Robert Zubek and Phillip Saltzman, Northwestern University
    email: rob@cs.northwestern.edu, p-saltzman@northwestern.edu

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

************************************************************************** */

#if _MSC_VER > 1000
#pragma once
#endif 

#ifndef SD_SHADOWDOOR_H
#define SD_SHADOWDOOR_H

#include "definitions.h"

//
//
// the main class

class ShadowDoor : public CNWNXBase {
public:

	/* functions */

	// creation handler
	virtual BOOL OnCreate (const char* LogDir = NULL);

	// request handler
	virtual char* OnRequest (char *gameObject, char* Request, char* Parameters);
	
	// release handler 
	virtual BOOL OnRelease ();


	// creates or retrieves the singleton instance of this class
	static inline ShadowDoor* GetInstance () { 
		if (singleton == NULL) singleton = new ShadowDoor();
		return singleton; 
	}

	// destroys singleton instance of the class
	static inline ShadowDoor DestroyInstance () { 
		delete singleton; 
	}

	// handles the socket->game message processing loop
	virtual void RunListener ();


	/* variables */

	// is anyone connected through the socket?
	bool bConnected;

	// should we dump a lot of info into the log?
	bool bVerbose;


protected:

	/* variables */ 

	// static reference to self
	static ShadowDoor* singleton;

	// queue of strings recently received from the socket, waiting for 
	// the game to read them
	MsgQueue fromSocketQueue;

	// map from string request to member function that handles it
	PointerTable handlers;

	// just a queue of string pointers to delete
	Msg stringsToDelete;

	// server socket, and connection socket
	SOCKET servSock;
	SOCKET externalSock;


	/* functions */

	// constructor and virtual destructor
	ShadowDoor ();
	virtual inline ~ShadowDoor ();

	// game message handlers. 
	// they are triggered by commands coming from NWNX
	char* HandleReadyP (char *body);
	char* HandleGetToken (char *body);
	char* HandleFinish (char *body);
	char* HandleSend (char *body);

	char* HandleSetVerbose (char *body); 
	char* HandleClearVerbose (char *body); 

};





//
//
// export declaration 

#ifdef SHADOWDOOR_EXPORTS
#define SHADOWDOOR_API extern "C" __declspec(dllexport)
#else
#define SHADOWDOOR_API extern "C" __declspec(dllimport)
#endif

SHADOWDOOR_API CNWNXBase* GetClassObject();


#endif // SD_SHADOWDOOR_H